#' A representation of a Domino DataSource
#'
#' @field id The ID of the data source
#' @field name The name of the data source
#' @field type The type of data source
#' @field config A list of config parameters
DominoDataSource <- R6::R6Class(
  "DominoDataSource",

  private = list(
    .id = NULL,
    .name = NULL,
    .type = NULL,
    .config = NULL
  ),

  active = list(
    id = function() {
      private$.id
    },
    name = function() {
      private$.name
    },
    type = function() {
      private$.type
    },
    config = function() {
      private$.config
    }
  ),

  public = list(

    #' @description
    #' Create a new instance
    #'
    #' @param id The ID of the data source
    #' @param name The name of the data source
    #' @param type The type of data source
    #' @param config A list of config parameters
    initialize = function(id, name, type, config = list()) {
      if (length(config) > 0 && (is.null(names(config)) || ("" %in% names(config)))) {
        stop("DominoDataSource: `config` must be a named list or an empty list", call. = FALSE)
      }
      if (is.null(id) || is.null(name) || is.null(type)) {
        stop("DominoDataSource: `id`, `name`, `type` must not be NULL")
      }
      private$.id <- id
      private$.name <- name
      private$.type <- type
      private$.config <- config
    },

    #' @description
    #' Print a string representation
    print = function() {
      main <- glue::glue("<DominoDataSource> { self$type } : { self$name } ({ self$id })")
      config <- ""
      if (length(self$config) > 0) {
        config <- paste(names(self$config), unname(self$config), sep = "=", collapse = "; ")
        config <- paste0("    Config: ", config, "\n")
      }
      cat0(main, '\n', config)
    }
  )
)
