#' A subclass of LoadComponent that loads a sample dataset
LoadComponentDemo <- R6::R6Class(
  "LoadComponentDemo",
  inherit = LoadComponent,

  private = list(
    .dataset = NULL,

    set_dataset = function(x) {
      if (!is.null(x) && x == "") {
        x <- NULL
      }
      private$.dataset <- x

      if (is.null(x)) {
        private$.name <- private$.code <- private$.data <- private$.error <- NULL
      } else {
        private$run()
      }
    },

    get_name = function() {
      private$.dataset
    },

    get_code = function() {
      if (!private$.dataset %in% LoadComponentDemo$DATASETS) {
        stop("Unknown dataset: ", private$.dataset)
      }
      if (private$.dataset %in% c("diamonds", "economics", "midwest")) {
        glue::glue("ggplot2::{ private$.dataset }")
      } else if (private$.dataset %in% c("iris", "mtcars")) {
        glue::glue("datasets::{ private$.dataset }")
      } else if (private$.dataset == "gapminder") {
        "gapminder::gapminder"
      } else if (private$.dataset %in% c("palmerpenguins", "stocks_wide", "stocks_long")) {
        glue::glue("{ PACKAGE_NAME }::{ private$.dataset }")
      } else if (private$.dataset %in% c("titanic", "small_molecule_drugbank")) {
        glue::glue('read.csv(system.file("sample_data", "{ private$.dataset }.csv", package = "{ PACKAGE_NAME }"))')
      }
    },

    get_data = function() {
      suppressWarnings(eval(parse(text = private$.code)))
    }
  ),

  public = list(

    #' @description
    #' Create a new instance
    #'
    #' @param dataset The name of the sample dataset to load
    initialize = function(dataset = NULL) {
      # If the dataset is the result of a failed shiny req() call, treat it as NULL instead of error
      dataset <- tryCatch(dataset, error = function(err) NULL)

      super$initialize()
      private$set_dataset(dataset)
      invisible(self)
    }
  )
)

LoadComponentDemo$DATASETS <- c("palmerpenguins", "gapminder", "iris", "mtcars", "diamonds", "stocks_wide",
                                "stocks_long", "titanic", "economics", "midwest", "small_molecule_drugbank")

#' Shiny module: Load a sample dataset
#'
#' @param id Unique ID for the module
#' @return List of reactives:
#'   - name
#'   - data
#'   - code
#'   - error
#' @name LoadComponentDemoShiny
LoadComponentDemo$shiny <- list(

  ui = function(id) {
    ns <- NS(id)
    tagList(
      br(),
      shinyfilebrowser::list_selector_ui(ns("datasets"))
    )
  },

  server = function(id) {
    moduleServer(
      id,
      function(input, output, session) {
        datasets <- shinyfilebrowser::list_selector_server(
          "datasets",
          LoadComponentDemo$DATASETS
        )
        load <- reactive({
          LoadComponentDemo$new(datasets())
        })

        return(list(
          name = reactive(load()$name),
          data = reactive(load()$data),
          code = reactive(load()$code),
          error = reactive(load()$error)
        ))
      }
    )
  }
)
