#' A subclass of LoadComponentFile that loads files by uploading a file
LoadComponentUpload <- R6::R6Class(
  "LoadComponentUpload",
  inherit = LoadComponentFile
)

#' Shiny module: Load a file by uploading a file
#'
#' @param id Unique ID for the module
#' @param upload_dir The directory where files get uploaded to
#' @return List of reactives:
#'   - name
#'   - data
#'   - code
#'   - error
#' @name LoadComponentUploadShiny
LoadComponentUpload$shiny <- list(

  ui = function(id) {
    ns <- NS(id)
    tagList(
      br(),
      shinyWidgets::alert(
        "Choose a file with an extension of:",
        paste(LoadComponentFile$FILE_READ_EXTENSIONS, collapse = ", "),
        status = "info", dismissible = TRUE
      ),
      fileInput(
        ns("file"), label = NULL, multiple = FALSE, width = "100%",
        accept = paste0(".", LoadComponentFile$FILE_READ_EXTENSIONS)
      ),
      load_file_params_ui(ns("params"))
    )
  },

  server = function(id, upload_dir = getwd()) {

    if (!dir.exists(upload_dir)) {
      dir.create(upload_dir, recursive = TRUE)
    }

    moduleServer(
      id,
      function(input, output, session) {
        new_path <- reactive({
          req(input$file)
          file.path(upload_dir, input$file$name)
        })

        observeEvent(input$file, {
          file.copy(input$file$datapath, new_path(), overwrite = TRUE)
        })

        load <- reactive({
          LoadComponentUpload$new(new_path(), params = params())
        })

        params <- load_file_params_server("params", file_path = reactive(input$file$name))

        return(list(
          name = reactive(load()$name),
          data = reactive(load()$data),
          code = reactive(load()$code),
          error = reactive(load()$error)
        ))
      }
    )
  }
)
