#' Get the name of a data source based on the `dataSourceType` field from Domino API
#'
#' @param type The data source type, taken from the `dataSourceType` field
get_data_source_shortname <- function(type) {
  if (is.null(type)) return()
  sub("Config$", "", type)
}

#' Whether or not a type of data source is supported
#'
#' @param dds A DominoDataSource object
is_data_source_supported <- function(dds) {
  stopifnot(inherits(dds, "DominoDataSource"))
  dds$type %in% ALL_DATASOURCES
}

#' Generate the HTML of a DataSource that should be displayed to represent it
#'
#' @param dds A DominoDataSource object
get_data_source_dropdown_html <- function(dds) {
  stopifnot(inherits(dds, "DominoDataSource"))
  if (is_data_source_supported(dds)) {
    img <- glue::glue("<img class='data-source-img' src='dca-assets/dca/img/datasource_{ tolower(dds$type) }.png' />")
  } else {
    img <- glue::glue("<div class='data-source-img'>&nbsp;</div>")
  }
  type <- dds$type
  if (!is_data_source_supported(dds)) {
    type <- paste(type, " [Not Supported]")
  }
  glue::glue("{img}&nbsp;&nbsp;{ dds$name } <small class='text-muted'>{ type }</small>")
}

ALL_DATASOURCES <- c(
  "S3",
  "GCS",
  "ADLS",
  "GenericS3",
  "PostgreSQL",
  "Snowflake",
  "Redshift",
  "SQLServer",
  "MySQL",
  "Oracle"
)
