# Domino Code Assist (DCA) for RStudio

<!-- badges: start -->
[![R-CMD-check](https://github.com/cerebrotech/domino-code-assist-rstudio/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/cerebrotech/domino-code-assist-rstudio/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

Accelerate routine data science tasks and smoothly interface with the Domino API through the DCA point-and-click GUI.

## Installation

### Method 1: Installing in a Domino RStudio workspace

Run the following commands inside RStudio:

```r
devtools::install_url('https://mirrors.domino.tech/dca/assistdomino_latest.tar.gz', upgrade = 'always')
```

This will install *DCA for RStudio* in your current workspace, but the tool will not persist once the workspace is stopped. 

*Note: If you encounter an error that includes "cannot open URL", make sure that you've selected a hardware tier for your Domino Workspace that allows egress. Otherwise, your Workspace will not be able to access the internet to download the installation file.*

### Method 2: Installing in a Domino Compute environment (Recommended)

1. Click on "Environments" on the side navigation bar
2. Select the environment in which *DCA for RStudio* should be installed
3. Click the "Edit Definition" button to allow you to make changes
4. Add the following two lines to the end of the Dockerfile section (but **before** the last `USER ubuntu` command):

    ```
    RUN R -e "devtools::install_url('https://mirrors.domino.tech/dca/assistdomino_latest.tar.gz', upgrade = 'always')"
    RUN chown -R ubuntu:ubuntu /usr/local/lib/R/site-library
    ```

5. Click the "Build" button at the bottom of the page

This will install *DCA for RStudio* in every workspace that uses the given Domino environment.

### Installing in an air-gapped environment 

If Method 1 or 2 work for you, this section can be skipped.

#### Background

The installation file for *DCA for RStudio* is available at [`https://mirrors.domino.tech/dca/assistdomino_latest.tar.gz`](https://mirrors.domino.tech/dca/assistdomino_latest.tar.gz). Generally, you need to install this file using `devtools::install_url()`. The installation process automatically downloads and installs any dependencies. However, if your Domino deployment does not have open access to the internet (ie. you're in an air-gapped environment), then any dependencies of DCA will fail to download. In this case, you will need to resort to an alternate method of installing DCA, either by pre-installing all dependencies or by having a CRAN-like repository on disk where the dependencies can be installed from.

#### How to install

Domino provides a CRAN-like repository to assist with installation. A zipped version of the repository can be downloaded from [`https://mirrors.domino.tech/dca/dca_sources.zip`](https://mirrors.domino.tech/dca/dca_sources.zip). This link will download a file `dca_sources.zip` of size ~100MB. After downloading this zip file, it will need to be unzipped into a `dca_sources` folder. This `dca_sources` folder needs to be made available in your Domino Project: it can be directly uploaded as a folder, placed in your organization's Git repository, or transferred using any other method that is available to you. 

Once the `dca_sources` folder is available in your Domino Project, it can be used to install DCA without internet in a similar fashion to Method 1 or 2 above (in both cases, you'll need to set `dir` in the code to the correct path).

1. To install using Method 1, use the following code:

    ```r
    dir <- '/path/to/dca_sources'
    install.packages('assistdomino', repos = NULL, contriburl = paste0('file://', dir, '/src/contrib'), type = 'source', upgrade = 'always')
    ```

2. To install using Method 2, run the same steps as Method 2 above, but use the following code in step 4:

    ```
    RUN R -e "dir <- '/path/to/dca_sources'; install.packages('assistdomino', repos = NULL, contriburl = paste0('file://', dir, '/src/contrib'), type = 'source', upgrade = 'always');"
    RUN chown -R ubuntu:ubuntu /usr/local/lib/R/site-library
    ```
    
    Remember that this will require the unzipped `dca_sources` path to exist in the Docker image.

Using the CRAN-like repository we provide is only one example of installing DCA in an air-gapped environment. If this solution cannot work in your deployment, please contact your IT department for help installing DCA and its dependencies.

## How to run

In RStudio, click the "Addins" menu.

![addins menu before click](inst/docs/screenshots/rstudio-addins-closed.png)

Under the heading of "ASSISTDOMINO" there will be a few buttons, one for each of the DCA modules: "DCA Data Selector", "DCA Data Transformation", "DCA Data Visualization". Click on any of the DCA modules to launch them.

> If you installed using Method 1 and you don't see the Domino Code Assist menus, you may need to click on the *Session* menu and choose *Terminate R* in order for RStudio to refresh.

![addins menu after click](inst/docs/screenshots/rstudio-addins-open.png)

## Feature flags

- If the environment variable `DCA_R_DATASOURCES_DISABLED` is set to anything other than an empty string, then the Data wizard will not show the Data Sources tab. 
- If the environment variable `DCA_R_SNIPPET_EDITING_DISABLED` is set to anything other than an empty string, snippet editing buttons will not be shown.
- The Domino config settings "mixpanelSettings::frontendClientEnabled" and "mixpanelSettings::backendClientEnabled" control whether or not Mixpanel is allowed to send tracking data via the client (browser) and backend (R API), respectively.

## Development notes

### Environment variables

When developing/testing, it's useful to add `DOMINO_USER_API_KEY` and `DOMINO_API_HOST` environment variables (eg. using `.Renviron` file). Other useful envvars can include `DOMINO_PROJECT_ID`, `DOMINO_IMPORTED_CODE_DIR`, `DOMINO_PROJ1_WORKING_DIR`, `SHINYMIXPANEL_TOKEN_OVERRIDE`

### Test coverage

To check test code coverage, run `covr::report()`. It can take ~10 minutes, so you can use the following to hear a beep when it finishes running:

```r
Sys.time(); covr::report(); Sys.time(); beepr::beep()
```

By default, {covr} skips any tests that have the `skip_on_cran()` flag. To make sure {covr} is fully testing everything, you should set the envvar `NOT_CRAN=true`.

### Publishing a new version

TLDR: Run the script in `inst/scripts/publish.R`. This will create two files `assistdomino_latest.tar.gz` and `dca_sources.zip`, which you then need to upload online for customers to access. Longer explanation follows:

When a new version is published, since the GitHub repository is private, it cannot be installed directly from GitHub (unless you have a GitHub PAT). Instead, the package needs to be built into a source tarball and hosted somewhere so that users can download it and install using `devtools::install_url()`. Currently, the file is hosted on Domino's mirrors website: [`https://mirrors.domino.tech/dca/assistdomino_latest.tar.gz`](https://mirrors.domino.tech/dca/assistdomino_latest.tar.gz).
 
For air-gapped installations, a CRAN-like repository needs to be created. This folder needs to be made available to customers who wish to install in air-gapped environments. Currently, this folder gets zipped and is available on Domino's mirrors website: [`https://mirrors.domino.tech/dca/dca_sources.zip`](https://mirrors.domino.tech/dca/dca_sources.zip).

### Connecting to Domino Data Sources during local dev

Data sources are only accessible via proxy objects that are only available inside deployments. If developing outside of a Domino deployment, you must either use mock data sources or follow the necessary steps to use real data sources.

#### To use mock data sources

- Set envvar `MOCK_DOMINODATAR` to a directory - this directory will contain the mock data sources.
- In this directory, create one directory for each data source, with the name of the folder being the name of the data source.
  - If this is an object store data source, you can place any files and folders inside.
  - If it's a database data source, then only `.csv` files will work (these csv files will mimic tables in the database).
- To use the sample mock datasources: `Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))`
- The steps above will mock the actual data source connections, but the list of data sources still comes from a Domino API call. To use the sample datasources list, run

    ```r
    mock_sources <- system.file("mock_datasources", "data_sources.json", package = "assistdomino")
    testthat::local_mocked_bindings(
      get_data_sources_raw = function(...) jsonlite::fromJSON(mock_sources, simplifyDataFrame = FALSE),
      .package = "assistdomino"
    )
    ```

#### To use real data sources

Running `DominoDataR::datasource_client()` will fail on your development machine. Here are some notes for enabling this on a Windows dev machine:

- Install miniconda `reticulate::install_miniconda()`
- Install domino's datasources python package `reticulate::py_install("dominodatalab-data", pip = TRUE, method = "virtualenv")` 
- Make sure this python is at the top of your PATH above other python installations (check the location using `reticulate::py_config()`)
- Make sure package is available `reticulate::py_module_available("domino_data")`
- Make sure the package can be loaded from python: open a python shell and run `from domino_data.data_sources import DataSourceClient` to ensure there are no errors. Right now trying to create a client object in python should not work: `client = DataSourceClient()`
- Install Teleport tsh client version 7.3.22 https://goteleport.com/docs/installation/  (replace the version with 7.3.22)
- Run `tsh login --proxy=dev-teleport.domino.tech:443` which will open a URL and should say login was successful. If not, you might need to request access to the cerebrotech's Engineering github team (https://github.com/orgs/cerebrotech/teams/engineers)
- Run `tsh kube login <deploy_id>` (deploy ID is for example `daattali12577`)
- Install kubectl https://kubernetes.io/docs/tasks/tools/install-kubectl-windows/ and make sure this installed version is what the PATH sees
- **Every time:** In two separate shells, run `kubectl port-forward -n domino-platform service/datasource-proxy 8034:80` and `kubectl port-forward -n domino-platform service/datasource-proxy 8080:8080`
- **Every time:** Add two environment variables `DOMINO_DATASOURCE_PROXY_FLIGHT_HOST=grpc://localhost:8080` and `DOMINO_DATASOURCE_PROXY_HOST=http://localhost:8034` (make sure the ports match the shell commands) (also make sure `MOCK_DOMINODATAR` envvar is unset)
- Because of a strange bug https://github.com/rstudio/reticulate/issues/1357 whenever I open RStudio, I need to first run `reticulate::import("domino_data.data_sources")`, let it fail, then restart the R session (ctrl+shift+F10), then run it again, and only then I'll be able to run `DominoDataR::datasource_client()`
