# Create a local CRAN-like repo to allow installing assistdomino in an air-gapped environment
# After running this script, DCA can be installed with:
# install.packages("assistdomino", repos = NULL, contriburl = paste0("file://", dir, "/src/contrib"), type = "source", upgrade = "always")

dir <- "dca_sources"
dir.create(dir, showWarnings = FALSE)

addPackageURL <- function(pkgName, url, ...) {
  exDir <- file.path(tempdir(), pkgName)
  if (dir.exists(exDir)) unlink(exDir, recursive = TRUE)
  tarball <- file.path(tempdir(), paste0(pkgName, ".tar.gz"))
  download.file(url, tarball, mode = "wb")
  untar(tarball, exdir = exDir)
  miniCRAN::addLocalPackage(pkgName, exDir, ..., build = TRUE)
}

addPackageCurrent <- function(pkgName, ...) {
  exDir <- file.path(tempdir(), pkgName)
  if (dir.exists(exDir)) unlink(exDir, recursive = TRUE)
  tarball <- devtools::build(path = tempdir())
  untar(tarball, exdir = exDir)
  miniCRAN::addLocalPackage("assistdomino", exDir, path = dir, build = TRUE)
}

addPackageGitHub <- function(repo, ...) {
  packageName <- basename(repo)
  exDir <- file.path(tempdir(), packageName)
  if (dir.exists(exDir)) unlink(exDir, recursive = TRUE)
  zipFile <- file.path(tempdir(), paste0(packageName, ".zip"))
  download.file(paste0("https://github.com/", repo, "/zipball/master"), zipFile, mode = "wb")
  unzip(zipFile, exdir = exDir)
  file.rename(list.files(exDir, full.names = TRUE)[1], file.path(exDir, packageName))
  miniCRAN::addLocalPackage(packageName, exDir, ..., build = TRUE)
}

options(pkg.platforms = "source")
deps_info <- pak::local_deps(".")

cran_pkgs <- deps_info[deps_info$type == "standard", ]$ref
if (!require("miniCRAN", quietly = TRUE)) install.packages("miniCRAN")
miniCRAN::makeRepo(cran_pkgs, path = dir, type = "source")

github_pkgs <- deps_info[deps_info$type == "github", ]$ref
nop <- sapply(github_pkgs, function(repo) {
  addPackageGitHub(repo, path = dir)
})
# make sure that none of the github packages have a dependency that’s not yet recorded in the CRAN packages
# If any github package has a new dependency, need to add it to cran_pkgs

other_pkgs <-  deps_info[!deps_info$type %in% c("standard", "github"), ]$ref
other_pkgs <- setdiff(other_pkgs, c("local::."))
nop <- sapply(other_pkgs, function(pkg) {
  message("Non-CRAN nor GitHub package: ", pkg)
})

# If I want to add {assistdomino} from Domino's mirrors site:
#url <- "https://mirrors.domino.tech/dca/assistdomino_latest.tar.gz"
#addPackageURL("assistdomino", url, path = dir)

# It I want to add {assistdomino} from local disk:
addPackageCurrent("assistdomino", path = dir)

zip("dca_sources.zip", files = list.files("dca_sources", recursive = TRUE, full.names = TRUE))
unlink("dca_sources", recursive = TRUE)
