% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DataSource-Database.R
\name{DatabaseDataSource}
\alias{DatabaseDataSource}
\title{A subclass of LoadComponentDataSource that represents a database connection}
\description{
A subclass must implement \code{private$get_full_info()} and can optionally
set \code{private$.uses_schema = FALSE}.
}
\section{Super classes}{
\code{\link[assistdomino:LoadComponent]{assistdomino::LoadComponent}} -> \code{\link[assistdomino:LoadComponentDataSource]{assistdomino::LoadComponentDataSource}} -> \code{DatabaseDataSource}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{database}}{The database selected}

\item{\code{schema}}{The schema selected}

\item{\code{table}}{The table selected}

\item{\code{uses_schema}}{Whether this database type uses the concept of a schema}

\item{\code{fixed_database}}{If the config provides a database, the user cannot change it}

\item{\code{fixed_schema}}{If the config provides a schema, the user cannot change it}

\item{\code{all_table_info}}{Dataframe with 4 columns: database, schema, table, rows}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-DatabaseDataSource-new}{\code{DatabaseDataSource$new()}}
\item \href{#method-DatabaseDataSource-print}{\code{DatabaseDataSource$print()}}
\item \href{#method-DatabaseDataSource-get_databases}{\code{DatabaseDataSource$get_databases()}}
\item \href{#method-DatabaseDataSource-set_database}{\code{DatabaseDataSource$set_database()}}
\item \href{#method-DatabaseDataSource-get_schemas}{\code{DatabaseDataSource$get_schemas()}}
\item \href{#method-DatabaseDataSource-set_schema}{\code{DatabaseDataSource$set_schema()}}
\item \href{#method-DatabaseDataSource-get_tables}{\code{DatabaseDataSource$get_tables()}}
\item \href{#method-DatabaseDataSource-set_table}{\code{DatabaseDataSource$set_table()}}
\item \href{#method-DatabaseDataSource-clone}{\code{DatabaseDataSource$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="assistdomino" data-topic="LoadComponentDataSource" data-id="reactive"><a href='../../assistdomino/html/LoadComponentDataSource.html#method-LoadComponentDataSource-reactive'><code>assistdomino::LoadComponentDataSource$reactive()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-new"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-new}{}}}
\subsection{Method \code{new()}}{
Create a new instance
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$new(dds)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{dds}}{The DominoDataSource associated with this database}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-print"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-print}{}}}
\subsection{Method \code{print()}}{
Print a string representation
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-get_databases"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-get_databases}{}}}
\subsection{Method \code{get_databases()}}{
Get a list of databases available in this datasource
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$get_databases()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-set_database"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-set_database}{}}}
\subsection{Method \code{set_database()}}{
Set the database
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$set_database(x)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{A database name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-get_schemas"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-get_schemas}{}}}
\subsection{Method \code{get_schemas()}}{
Get a list of schemas available for the given database
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$get_schemas()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-set_schema"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-set_schema}{}}}
\subsection{Method \code{set_schema()}}{
Set the schema
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$set_schema(x)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{A schema name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-get_tables"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-get_tables}{}}}
\subsection{Method \code{get_tables()}}{
Get a list of tables available for the given database/schema
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$get_tables()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-set_table"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-set_table}{}}}
\subsection{Method \code{set_table()}}{
Set the table
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$set_table(x)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{A table name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DatabaseDataSource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-DatabaseDataSource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DatabaseDataSource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
