% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformation.R
\name{Transformation}
\alias{Transformation}
\title{R6 superclass for storing information about a transformation}
\description{
A subclass must implement:
\itemize{
\item \code{private$get_full_code(name_in)}
\item \code{private$get_dependencies()}
}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{name_out}}{The name of the resulting dataframe}

\item{\code{tidyverse}}{Whether this transformation should use tidyverse code or base R}

\item{\code{dependencies}}{List of dependency packages used by the code in this transformation}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Transformation-new}{\code{Transformation$new()}}
\item \href{#method-Transformation-print}{\code{Transformation$print()}}
\item \href{#method-Transformation-use_tidyverse}{\code{Transformation$use_tidyverse()}}
\item \href{#method-Transformation-get_code}{\code{Transformation$get_code()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Transformation-new"></a>}}
\if{latex}{\out{\hypertarget{method-Transformation-new}{}}}
\subsection{Method \code{new()}}{
Create a new instance
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Transformation$new(name_out = NULL, tidyverse = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name_out}}{The name of the resulting dataframe}

\item{\code{tidyverse}}{Whether this transformation should use tidyverse code or base R}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Transformation-print"></a>}}
\if{latex}{\out{\hypertarget{method-Transformation-print}{}}}
\subsection{Method \code{print()}}{
Print a string representation
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Transformation$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Transformation-use_tidyverse"></a>}}
\if{latex}{\out{\hypertarget{method-Transformation-use_tidyverse}{}}}
\subsection{Method \code{use_tidyverse()}}{
Whether this transformation should use tidyverse code or base R
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Transformation$use_tidyverse(use)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{use}}{A boolean whether to use tidyverse or not}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Transformation-get_code"></a>}}
\if{latex}{\out{\hypertarget{method-Transformation-get_code}{}}}
\subsection{Method \code{get_code()}}{
Get the code of the transformation as a single string
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Transformation$get_code(name_in, dependencies = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name_in}}{The name of the input dataframe}

\item{\code{dependencies}}{Whether or not to include \code{library()} calls}
}
\if{html}{\out{</div>}}
}
}
}
