% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TransformationSequence.R
\name{TransformationSequence}
\alias{TransformationSequence}
\title{Ordered sequence of \code{Transformation} objects}
\description{
\code{TransformationSequence} is a data structure that holds an ordered list of
transformations. It does not hold the data on which the transformations
operate, only the transformations themselves and the name of the initial data.
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{name_in}}{The name of the input dataframe}

\item{\code{name_out}}{The name of the resulting dataframe after the transformations are applied}

\item{\code{transformations}}{The list of \code{Transformation} objects}

\item{\code{size}}{The number of transformations}

\item{\code{tidyverse}}{Whether the entire sequence has an override of using/not using tidyverse}

\item{\code{dependencies}}{Vector of dependency packages used in any of the transformations}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-TransformationSequence-new}{\code{TransformationSequence$new()}}
\item \href{#method-TransformationSequence-print}{\code{TransformationSequence$print()}}
\item \href{#method-TransformationSequence-use_tidyverse}{\code{TransformationSequence$use_tidyverse()}}
\item \href{#method-TransformationSequence-get_code}{\code{TransformationSequence$get_code()}}
\item \href{#method-TransformationSequence-get_setup_chunks}{\code{TransformationSequence$get_setup_chunks()}}
\item \href{#method-TransformationSequence-get_code_chunks}{\code{TransformationSequence$get_code_chunks()}}
\item \href{#method-TransformationSequence-add}{\code{TransformationSequence$add()}}
\item \href{#method-TransformationSequence-insert}{\code{TransformationSequence$insert()}}
\item \href{#method-TransformationSequence-remove}{\code{TransformationSequence$remove()}}
\item \href{#method-TransformationSequence-head}{\code{TransformationSequence$head()}}
\item \href{#method-TransformationSequence-run}{\code{TransformationSequence$run()}}
\item \href{#method-TransformationSequence-clone}{\code{TransformationSequence$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-new"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-new}{}}}
\subsection{Method \code{new()}}{
Create a new instance
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$new(transformations = list(), name_in, tidyverse = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{transformations}}{A list of \code{Transformation} objects}

\item{\code{name_in}}{The name of the input dataframe}

\item{\code{tidyverse}}{If \code{TRUE} or \code{FALSE}, override all transformations with this value.
If \code{NULL}, each transformation can have its own tidyverse status.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-print"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-print}{}}}
\subsection{Method \code{print()}}{
Print a string representation
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-use_tidyverse"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-use_tidyverse}{}}}
\subsection{Method \code{use_tidyverse()}}{
Override all transformations to use/not use tidyverse
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$use_tidyverse(use)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{use}}{If \code{TRUE} or \code{FALSE}, override all transformations with this value.
If \code{NULL}, each transformation can have its own tidyverse status.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-get_code"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-get_code}{}}}
\subsection{Method \code{get_code()}}{
Get the code of the transformations as a single string
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$get_code(include_setup = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{include_setup}}{Whether to include \code{library()} calls before the code}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-get_setup_chunks"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-get_setup_chunks}{}}}
\subsection{Method \code{get_setup_chunks()}}{
Get a list of \code{library()} calls for dependencies in any of the transformations
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$get_setup_chunks()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-get_code_chunks"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-get_code_chunks}{}}}
\subsection{Method \code{get_code_chunks()}}{
Get the code of the transformations as a list of chunks
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$get_code_chunks(include_setup = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{include_setup}}{Whether to include \code{library()} calls before the code}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-add"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-add}{}}}
\subsection{Method \code{add()}}{
Add a transformation (as the last one in the sequence)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$add(transformation)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{transformation}}{A \code{Transformation} to add}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-insert"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-insert}{}}}
\subsection{Method \code{insert()}}{
Add a transformation in an arbitrary location
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$insert(transformation, after)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{transformation}}{A \code{Transformation} to add}

\item{\code{after}}{The index after which to add this transformation}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-remove"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-remove}{}}}
\subsection{Method \code{remove()}}{
Remove a transformation
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$remove(n)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n}}{The index of a transformation to remove}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-head"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-head}{}}}
\subsection{Method \code{head()}}{
Get the first n transformations
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$head(n)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n}}{The number of transformations to retrieve}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-run"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-run}{}}}
\subsection{Method \code{run()}}{
Run the sequence of transformations
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$run(env = parent.frame(), data_in = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{env}}{The environment in which to execute the code}

\item{\code{data_in}}{(optional) A dataframe to use as the initial input}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TransformationSequence-clone"></a>}}
\if{latex}{\out{\hypertarget{method-TransformationSequence-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TransformationSequence$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
