library(shiny)

skip_on_cran()

test_that("LoadComponentDatasets module works", {
  withr::local_envvar(list(DOMINO_DATASETS_DIR = system.file("tests_data", "load_data", package = "assistdomino")))
  app <- shinyApp(
    fluidPage(assistdomino:::LoadComponentDatasets$shiny$ui("test")),
    function(input, output, session) {
      res <- assistdomino:::LoadComponentDatasets$shiny$server("test")
      exportTestValues(
        name = res$name(),
        code = if (is.null(res$code())) NULL else gsub("\\(.*\\)", "(<file>)", res$code()),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "LoadComponentDatasets")

  driver$set_inputs(
    `test-filebrowser-file_clicked` = file.path(get_user_datasets_dir(), "cars.csv"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  expect_equal(driver$get_value(export = "data"), cars)
  expect_true(is_shiny_visible(driver, "test-params-header"))
  expect_true(is_shiny_visible(driver, "test-params-sep"))
  expect_true(is_shiny_visible(driver, "test-params-dec"))
  expect_false(is_shiny_visible(driver, "test-params-col_names"))
  expect_false(is_shiny_visible(driver, "test-params-sheet"))

  driver$set_inputs(
    `test-filebrowser-file_clicked` = file.path(get_user_datasets_dir(), "cars.csv"),
    allow_no_input_binding_ = TRUE, priority_ = "event",
  )
  driver$wait_for_idle()
  driver$expect_values(export = TRUE)
  expect_false(is_shiny_visible(driver, "test-params-header"))
  expect_false(is_shiny_visible(driver, "test-params-sep"))
  expect_false(is_shiny_visible(driver, "test-params-dec"))
  expect_false(is_shiny_visible(driver, "test-params-col_names"))
  expect_false(is_shiny_visible(driver, "test-params-sheet"))

  driver$set_inputs(
    `test-filebrowser-file_clicked` = file.path(get_user_datasets_dir(), "cars.xlsx"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  expect_equal(driver$get_value(export = "data"), cars, ignore_attr = TRUE)
  expect_false(is_shiny_visible(driver, "test-params-header"))
  expect_false(is_shiny_visible(driver, "test-params-sep"))
  expect_false(is_shiny_visible(driver, "test-params-dec"))
  expect_true(is_shiny_visible(driver, "test-params-col_names"))
  expect_true(is_shiny_visible(driver, "test-params-sheet"))

  driver$set_inputs("test-params-sheet" = 2)
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  df <- cars
  names(df) <- paste0("...", c(cars[1, 1], cars[1, 2]))
  df <- df[-1, ]
  expect_equal(driver$get_value(export = "data"), df, ignore_attr = TRUE)

  driver$set_inputs("test-params-col_names" = FALSE)
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  df <- cars
  names(df) <- c("...1", "...2")
  expect_equal(driver$get_value(export = "data"), df, ignore_attr = TRUE)

  driver$set_inputs(
    `test-filebrowser-file_clicked` = file.path(get_user_datasets_dir(), "cars_wrong_format.sas7bdat"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  expect_null(driver$get_value(export = "data"))
  expect_null(driver$get_value(export = "name"))
  expect_match(driver$get_value(export = "error"), "Could not load data")
  expect_false(is_shiny_visible(driver, "test-params-header"))
  expect_false(is_shiny_visible(driver, "test-params-sep"))
  expect_false(is_shiny_visible(driver, "test-params-dec"))
  expect_false(is_shiny_visible(driver, "test-params-col_names"))
  expect_false(is_shiny_visible(driver, "test-params-sheet"))

  driver$set_inputs(
    `test-filebrowser-file_clicked` = file.path(get_user_datasets_dir(), "cars_semicolon.txt"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  df <- data.frame("speed.dist" = paste0(cars$speed, ";", cars$dist))
  expect_equal(driver$get_value(export = "data"), df)
  expect_true(is_shiny_visible(driver, "test-params-header"))
  expect_true(is_shiny_visible(driver, "test-params-sep"))
  expect_true(is_shiny_visible(driver, "test-params-dec"))
  expect_false(is_shiny_visible(driver, "test-params-col_names"))
  expect_false(is_shiny_visible(driver, "test-params-sheet"))

  driver$set_inputs("test-params-sep" = ";")
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  expect_equal(driver$get_value(export = "data"), cars)

  driver$set_inputs(
    `test-filebrowser-file_clicked` = file.path(get_user_datasets_dir(), "cars.sav"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  expect_equal(driver$get_value(export = "data"), cars, ignore_attr = TRUE)
  expect_false(is_shiny_visible(driver, "test-params-header"))
  expect_false(is_shiny_visible(driver, "test-params-sep"))
  expect_false(is_shiny_visible(driver, "test-params-dec"))
  expect_false(is_shiny_visible(driver, "test-params-col_names"))
  expect_false(is_shiny_visible(driver, "test-params-sheet"))

  driver$set_inputs(
    `test-filebrowser-file_clicked` = file.path(get_user_datasets_dir(), "cars.rds"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code", "error"))
  expect_equal(driver$get_value(export = "data"), cars, ignore_attr = TRUE)
  expect_false(is_shiny_visible(driver, "test-params-header"))
  expect_false(is_shiny_visible(driver, "test-params-sep"))
  expect_false(is_shiny_visible(driver, "test-params-dec"))
  expect_false(is_shiny_visible(driver, "test-params-col_names"))
  expect_false(is_shiny_visible(driver, "test-params-sheet"))
})
