library(shiny)

skip_on_cran()

test_that("Database data source shiny works - empty config", {
  TestDatabaseDataSource <- R6::R6Class(
    "TestDatabaseDataSource",
    inherit = assistdomino:::DatabaseDataSource,
    private = list(
      get_full_info = function() {
        data.frame(
          database = c("db1", "db1", "db1", "db1", "db2", "db2"),
          schema = c("schema1", "schema1", "schema1", "schema2", "schema2", "schema2"),
          table = c("diamonds", "snow1", "snow2", "snow1", "snow2", "notable"),
          rows = c(53940, 2, 2, 2, 2, 2)
        )
      }
    )
  )

  app <- shinyApp(
    fluidPage(assistdomino:::database_datasource_ui("test")),
    function(input, output, session) {
      Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))

      dds <- assistdomino:::DominoDataSource$new("id", "snowflake", "Test")
      ds <- TestDatabaseDataSource$new(dds)
      res <- assistdomino:::database_datasource_server("test", ds)

      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "datasource-database-shiny-noconfig")
  driver$wait_for_idle()

  driver$set_inputs(`test-database` = "db1")
  driver$wait_for_idle()
  expect_null_exports(driver)
  driver$set_inputs(`test-schema` = "schema1")
  driver$wait_for_idle()
  expect_null_exports(driver)
  driver$set_inputs(`test-table` = "diamonds")
  driver$wait_for_idle()
  expect_null_exports(driver, "error")
  expect_identical(driver$get_value(export = "name"), "diamonds")
  expect_identical(dim(driver$get_value(export = "data")), c(53940L, 2L))
  driver$expect_values(export = "code")
  driver$set_inputs(`test-table` = "snow1")
  driver$wait_for_idle()
  expect_null_exports(driver, "error")
  expect_identical(driver$get_value(export = "name"), "snow1")
  driver$expect_values(export = c("code", "data"))
  driver$set_inputs(`test-schema` = "schema2")
  driver$wait_for_idle()
  expect_null_exports(driver)
  driver$set_inputs(`test-table` = "snow1")
  driver$wait_for_idle()
  expect_null_exports(driver, "error")
  expect_identical(driver$get_value(export = "name"), "snow1")
  driver$expect_values(export = c("code", "data"))
  driver$set_inputs(`test-database` = "db2")
  driver$wait_for_idle()
  expect_null_exports(driver)
  driver$set_inputs(`test-schema` = "schema2")
  driver$wait_for_idle()
  expect_null_exports(driver)
  driver$set_inputs(`test-table` = "snow2")
  driver$wait_for_idle()
  expect_null_exports(driver, "error")
  expect_identical(driver$get_value(export = "name"), "snow2")
  driver$expect_values(export = c("code", "data"))
  driver$set_inputs(`test-table` = "notable")
  driver$wait_for_idle()
  expect_null_exports(driver, c("code", "name", "data"))
  expect_match(driver$get_value(export = "error"), "Could not load data.*notable.*doesn't exist")
})

test_that("Database data source shiny works - config has database and schema", {
  TestDatabaseDataSource <- R6::R6Class(
    "TestDatabaseDataSource",
    inherit = assistdomino:::DatabaseDataSource,
    private = list(
      get_full_info = function() {
        data.frame(
          database = c("db1", "db1", "db1", "db1", "db2", "db2"),
          schema = c("schema1", "schema1", "schema1", "schema2", "schema2", "schema2"),
          table = c("diamonds", "snow1", "snow2", "snow1", "snow2", "notable"),
          rows = c(53940, 2, 2, 2, 2, 2)
        )
      }
    )
  )

  app <- shinyApp(
    fluidPage(assistdomino:::database_datasource_ui("test")),
    function(input, output, session) {
      Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))

      dds <- assistdomino:::DominoDataSource$new("id", "snowflake", "Test", list(database = "db2", schema = "schema2"))
      ds <- TestDatabaseDataSource$new(dds)
      res <- assistdomino:::database_datasource_server("test", ds)

      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "datasource-database-shiny-dbschema")
  driver$wait_for_idle()
  expect_null_exports(driver)
  driver$set_inputs(`test-table` = "notable")
  driver$wait_for_idle()
  expect_null_exports(driver, c("code", "name", "data"))
  expect_match(driver$get_value(export = "error"), "Could not load data.*notable.*doesn't exist")
  driver$set_inputs(`test-table` = "snow2")
  driver$wait_for_idle()
  expect_null_exports(driver, "error")
  expect_identical(driver$get_value(export = "name"), "snow2")
  driver$expect_values(export = c("code", "data"))
})

test_that("Database data source shiny works - database doesn't use schema", {
  TestDatabaseDataSource <- R6::R6Class(
    "TestDatabaseDataSource",
    inherit = assistdomino:::DatabaseDataSource,
    private = list(
      .uses_schema = FALSE,
      get_full_info = function() {
        data.frame(
          database = c("db1", "db1", "db1", "db2", "db2"),
          table = c("diamonds", "snow1", "snow2", "snow2", "notable"),
          rows = c(53940, 2, 2, 2, 2)
        )
      }
    )
  )

  app <- shinyApp(
    fluidPage(assistdomino:::database_datasource_ui("test")),
    function(input, output, session) {
      Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))

      dds <- assistdomino:::DominoDataSource$new("id", "snowflake", "Test")
      ds <- TestDatabaseDataSource$new(dds)
      res <- assistdomino:::database_datasource_server("test", ds)

      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "datasource-database-shiny-no-uses-schema-no-config")
  driver$wait_for_idle()
  driver$set_inputs(`test-database` = "db1")
  driver$wait_for_idle()
  expect_null_exports(driver)
  driver$set_inputs(`test-table` = "snow1")
  driver$wait_for_idle()
  expect_null_exports(driver, "error")
  expect_identical(driver$get_value(export = "name"), "snow1")
  driver$expect_values(export = c("code", "data"))

  app <- shinyApp(
    fluidPage(assistdomino:::database_datasource_ui("test")),
    function(input, output, session) {
      Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))

      dds <- assistdomino:::DominoDataSource$new("id", "snowflake", "Test", list(database = "db1"))
      ds <- TestDatabaseDataSource$new(dds)
      res <- assistdomino:::database_datasource_server("test", ds)

      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "datasource-database-shiny-no-uses-schema-config-db")
  driver$wait_for_idle()
  driver$set_inputs(`test-table` = "snow1")
  driver$wait_for_idle()
  expect_null_exports(driver, "error")
  expect_identical(driver$get_value(export = "name"), "snow1")
  driver$expect_values(export = c("code", "data"))
})
