test_that("Database data source works", {
  TestDatabaseDataSource <- R6::R6Class(
    "TestDatabaseDataSource",
    inherit = DatabaseDataSource,
    private = list(
      get_full_info = function() {
        data.frame(
          database = c("db1", "db1", "db1", "db1", "db2"),
          schema = c("schema1", "schema1", "schema1", "schema2", "schema2"),
          table = c("diamonds", "snow1", "snow2", "snow1", "snow2"),
          rows = c(53940, 2, 2, 2, 2)
        )
      }
    )
  )
  withr::local_envvar(c("MOCK_DOMINODATAR" = system.file("mock_datasources", package = PACKAGE_NAME)))

  dds <- DominoDataSource$new("id", "snowflake", "Test")
  db <- TestDatabaseDataSource$new(dds)
  expect_null(db$code)
  expect_null(db$name)
  expect_null(db$data)
  expect_null(db$error)
  expect_null(db$database)
  expect_null(db$schema)
  expect_null(db$table)
  expect_false(db$fixed_database)
  expect_false(db$fixed_schema)
  expect_identical(db$get_databases(), c("db1", "db2"))
  expect_length(db$get_schemas(), 0)
  expect_identical(nrow(db$get_tables()), 0L)

  db$set_database("")
  expect_null(db$database)
  expect_length(db$get_schemas(), 0)
  expect_null(db$error)

  db$set_database("baddb")
  expect_identical(db$database, "baddb")
  expect_length(db$get_schemas(), 0)
  expect_null(db$error)

  db$set_database("db1")
  expect_identical(db$database, "db1")
  expect_identical(db$get_schemas(), c("schema1", "schema2"))
  expect_identical(nrow(db$get_tables()), 0L)
  expect_null(db$error)

  db$set_schema("badschema")
  expect_identical(nrow(db$get_tables()), 0L)
  expect_null(db$error)

  db$set_schema("schema1")
  expect_identical(db$schema, "schema1")
  expect_identical(nrow(db$get_tables()), 3L)
  expect_null(db$table)
  expect_null(db$code)
  expect_null(db$error)

  db$set_table("table1")
  expect_identical(db$table, "table1")
  expect_null(db$code)
  expect_match(db$error, "Could not load data.*table1.*doesn't exist")

  db$set_table("snow1")
  expect_identical(db$table, "snow1")
  expect_null(db$error)
  expect_identical(db$name, "snow1")
  expect_identical(dim(db$data), c(2L, 2L))
  expect_identical(names(db$data), c("snowcol1", "snowcol2"))
  expect_match(db$code, "client <-.*datasource_client().*SELECT \\* FROM db1.schema1.snow1.*query")

  db$set_database("db2")
  db$set_schema("schema2")
  db$set_table("snow2")
  expect_null(db$error)
  expect_identical(db$name, "snow2")
  expect_identical(dim(db$data), c(2L, 2L))
  expect_identical(names(db$data), c("SNOWcol1", "SNOWcol2"))
  expect_match(db$code, "client <-.*datasource_client().*SELECT \\* FROM db2.schema2.snow2.*query")
  expect_output(db$print(), "TestDatabaseDataSource.*Test.*snowflake.*db2.*schema2.*snow2")

  dds <- DominoDataSource$new("id", "badname", "Test")
  db <- TestDatabaseDataSource$new(dds)
  db$set_database("db1")
  db$set_schema("schema1")
  expect_null(db$error)
  db$set_table("snow1")
  expect_match(db$error, "Could not load data.*datasource folder doesn't exist")

  dds <- DominoDataSource$new("id", "snowflake", "Test", config = list(database = "db1"))
  db <- TestDatabaseDataSource$new(dds)
  expect_null(db$code)
  expect_null(db$name)
  expect_null(db$data)
  expect_null(db$error)
  expect_identical(db$database, "db1")
  expect_null(db$schema)
  expect_null(db$table)
  expect_true(db$fixed_database)
  expect_false(db$fixed_schema)
  expect_identical(db$get_schemas(), c("schema1", "schema2"))
  expect_identical(nrow(db$get_tables()), 0L)

  db$set_schema("schema1")
  expect_identical(nrow(db$get_tables()), 3L)
  expect_null(db$table)
  expect_null(db$code)
  expect_null(db$error)

  db$set_schema("schema2")
  expect_identical(nrow(db$get_tables()), 1L)
  expect_null(db$table)
  expect_null(db$code)
  expect_null(db$error)
  db$set_table("snow1")
  expect_null(db$error)
  expect_identical(db$table, "snow1")
  expect_match(db$code, "client <-.*datasource_client().*SELECT \\* FROM schema2.snow1.*query")

  dds <- DominoDataSource$new("id", "snowflake", "Test", config = list(database = "db1", schema = "schema1"))
  db <- TestDatabaseDataSource$new(dds)
  expect_identical(db$database, "db1")
  expect_identical(db$schema, "schema1")
  expect_null(db$table)
  expect_true(db$fixed_database)
  expect_true(db$fixed_schema)
  expect_identical(nrow(db$get_tables()), 3L)
  expect_null(db$error)
  db$set_table("snow2")
  expect_match(db$code, "client <-.*datasource_client().*SELECT \\* FROM snow2.*query")

  db$set_table("diamonds")
  expect_match(db$code, "client <-.*datasource_client().*SELECT \\* FROM diamonds LIMIT 5000.*query")
})
