library(shiny)

skip_on_cran()

test_that("ObjectStore data source shiny works", {
  app <- shinyApp(
    fluidPage(assistdomino:::object_store_datasource_ui("test")),
    function(input, output, session) {
      Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))

      dds <- assistdomino:::DominoDataSource$new("id", "badsource", "S3")
      ds <- assistdomino:::S3DataSource$new(dds)
      res <- assistdomino:::object_store_datasource_server("test", ds)

      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE)
  driver$wait_for_idle()
  driver$wait_for_js('$("#test-error").text().includes("datasource folder doesn\'t exist")')

  app <- shinyApp(
    fluidPage(assistdomino:::object_store_datasource_ui("test")),
    function(input, output, session) {
      Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))

      dds <- assistdomino:::DominoDataSource$new("id", "s3", "S3")
      ds <- assistdomino:::S3DataSource$new(dds)
      res <- assistdomino:::object_store_datasource_server("test", ds)

      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "datasource-objectstore-shiny")
  driver$wait_for_idle()
  expect_identical(driver$get_text("#test-error"), "")

  driver$set_inputs(
    `test-files-file_clicked` = "s3file.csv",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = TRUE)

  driver$set_inputs(
    `test-files-file_clicked` = "s3file.csv",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = TRUE)

  driver$set_inputs(
    `test-files-file_clicked` = "badfile.csv",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = TRUE)

 

  app <- shinyApp(
    fluidPage(assistdomino:::object_store_datasource_ui("test")),
    function(input, output, session) {
      Sys.setenv("MOCK_DOMINODATAR" = system.file("mock_datasources", package = "assistdomino"))

      dds <- assistdomino:::DominoDataSource$new("id", "gcs", "S3")
      ds <- assistdomino:::S3DataSource$new(dds)
      res <- assistdomino:::object_store_datasource_server("test", ds)

      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data(),
        error = res$error()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE)
  driver$wait_for_idle()
  driver$wait_for_js('$("#test-files-container").text().includes("no supported files")')
})
