test_that("Object store data source get_files works", {
  withr::local_envvar(c("MOCK_DOMINODATAR" = system.file("mock_datasources", package = PACKAGE_NAME)))

  expect_error(
    S3DataSource$new(DominoDataSource$new("id", "badname", "S3"))$get_files(),
    "datasource folder doesn't exist"
  )

  expect_length(
    GCSDataSource$new(DominoDataSource$new("id", "gcs", "GCS"))$get_files(),
    0
  )

  expect_length(
    S3DataSource$new(DominoDataSource$new("id", "s3", "S3"))$get_files(),
    3
  )
})

test_that("Object store data source set_file works", {
  withr::local_envvar(c("MOCK_DOMINODATAR" = system.file("mock_datasources", package = PACKAGE_NAME)))

  s3bad <- S3DataSource$new(DominoDataSource$new("id", "badname", "S3"))$set_file("s3file.csv")
  expect_match(s3bad$error, "datasource folder doesn't exist")
  expect_null(s3bad$code)
  expect_null(s3bad$name)
  expect_null(s3bad$data)

  s3nofile <- S3DataSource$new(DominoDataSource$new("id", "s3", "S3"))$set_file("nofile.csv")
  expect_match(s3nofile$error, "object doesn't exist")
  expect_null(s3nofile$code)
  expect_null(s3nofile$name)
  expect_null(s3nofile$data)

  withr::with_tempdir({
    s3good <- S3DataSource$new(DominoDataSource$new("id", "s3", "S3"))$set_file("s3file.csv")
    expect_null(s3good$error)
    expect_match(s3good$code, "client <-.*datasource_client().*s3file.csv.*save_object.*read.csv")
    expect_identical(s3good$name, "s3file")
    expect_identical(dim(s3good$data), c(2L, 2L))
  })
})
