library(shiny)

skip_on_cran()

test_that("data_environment module works when there are no dataframes", {
  app <- shinyApp(
    fluidPage(assistdomino:::data_environment_ui("test")),
    function(input, output, session) {
      res <- assistdomino:::data_environment_server("test")
      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE)

  driver$wait_for_value(output = "test-dataframes-file_list")
  expect_match(driver$get_text("#test-dataframes-file_list"), "no data.frames")
})

test_that("data_environment module works", {
  app <- shinyApp(
    fluidPage(assistdomino:::data_environment_ui("test")),
    function(input, output, session) {
      assign("df1", cars, envir = globalenv())
      assign("df2", iris, envir = globalenv())

      res <- assistdomino:::data_environment_server("test")
      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "data-environment")

  driver$wait_for_idle()
  driver$expect_values(export = TRUE)

  driver$set_inputs(
    `test-dataframes-file_clicked` = "df1",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code"))
  expect_equal(driver$get_value(export = "data"), cars)

  driver$set_inputs(
    `test-dataframes-file_clicked` = "df2",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = c("name", "code"))
  expect_equal(driver$get_value(export = "data"), iris)

  driver$set_inputs(
    `test-dataframes-file_clicked` = "df2",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = TRUE)
})

test_that("data_environment module works with df_quick_start", {
  skip_on_ci()

  app <- shinyApp(
    fluidPage(assistdomino:::data_environment_ui("test")),
    function(input, output, session) {
      res <- assistdomino:::data_environment_server("test", demo = TRUE)
      exportTestValues(
        name = res$name(),
        code = res$code(),
        data = res$data()
      )
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE)

  driver$wait_for_value(output = "test-dataframes-file_list")
  expect_match(driver$get_text("#test-dataframes-file_list"), "df_quick_start")
})
