library(shiny)

skip_on_cran()

test_that("page_data_select shiny module works", {
  app <- shinyApp(
    ui = fluidPage(
      assistdomino:::page_data_select_ui("test")
    ),
    server = function(input, output, session) {
      Sys.setenv(SHINYMIXPANEL_DISABLE = "1")

      res <- assistdomino:::page_data_select_server("test")
      exportTestValues(
        name = res$name(),
        data = res$data()
      )
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE)
  driver$wait_for_idle()

  wait_for_visible(driver, "test-continue")
  wait_for_disabled(driver, "test-continue")

  driver$set_inputs(`test-import_modules` = "demo")
  driver$set_inputs(
    `test-demo-datasets-file_clicked` = "mtcars",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  wait_for_enabled(driver, "test-continue")
  expect_equal(driver$get_value(export = "name"), "df")
  expect_equal(driver$get_value(export = "data"), mtcars)

  expect_true(is_shiny_visible(driver, "test-preview_data"))
  driver$set_inputs(`test-show_preview` = FALSE)
  expect_false(is_shiny_visible(driver, "test-preview_data"))

  expect_true(is_shiny_visible(driver, "test-code-code_section"))
  driver$set_inputs(`test-show_code` = FALSE)
  expect_false(is_shiny_visible(driver, "test-code-code_section"))

  driver$set_inputs(`test-varname` = "df car")
  driver$wait_for_idle()
  expect_equal(driver$get_value(input = "test-varname"), "df.car")
  expect_equal(driver$get_value(export = "name"), "df.car")

  driver$set_inputs(`test-custom_name` = FALSE)
  driver$wait_for_idle()
  expect_equal(driver$get_value(export = "name"), "mtcars")

   driver$set_inputs(
    `test-demo-datasets-file_clicked` = "iris",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  expect_equal(driver$get_value(export = "name"), "iris")
  expect_equal(driver$get_value(export = "data"), iris)

  driver$set_inputs(`test-custom_name` = TRUE)
  driver$wait_for_idle()
  expect_equal(driver$get_value(export = "name"), "df.car")

  expect_true(is_shiny_enabled(driver, "test-continue"))
  driver$set_inputs(`test-import_modules` = "upload")
  wait_for_disabled(driver, "test-continue")

  driver$set_inputs(`test-import_modules` = "demo")
  wait_for_enabled(driver, "test-continue")

  driver$set_inputs(`test-import_modules` = "url")
  wait_for_disabled(driver, "test-continue")
  driver$get_text("#test-error")
  driver$set_inputs(`test-url-url` = "badurl")
  driver$wait_for_idle()
  expect_match(driver$get_text("#test-error"), "Could not load data")
})
