library(shiny)

skip_on_cran()

test_that("page_xforms shiny module works when not given a dataset", {
  mtcarsnew <- mtcars
  mtcarsnew[c(1:10), "am"] <- NA

  app <- shinyApp(
    ui = fluidPage(
      assistdomino:::page_xforms_ui("test")
    ),
    server = function(input, output, session) {
      Sys.setenv(SHINYMIXPANEL_DISABLE = "1")

      assign("df1", mtcarsnew, envir = globalenv())

      res <- assistdomino:::page_xforms_server("test")
      exportTestValues(
        code = res$code(),
        result = res$result()
      )
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "page-xforms")
  driver$wait_for_idle()

  wait_for_visible(driver, "test-data_select")
  driver$set_inputs(
    `test-data_select_mod-dataframes-file_clicked` = "df1",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  wait_for_visible(driver, "test-main_section")
  expect_equal(driver$get_value(export = "code"), "df1")
  expect_identical(driver$get_value(export = "result"), mtcarsnew)

  driver$set_inputs(
    `test-table-drop` = "disp",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, select = -disp)
  )

  driver$set_inputs(
    `test-table-filter` = list(col = "cyl", val = "4"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  wait_for_visible(driver, "test-xform_modal-apply")
  driver$click("test-xform_modal-apply")
  wait_for_not_exists(driver, "test-xform_modal-apply")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, subset = cyl == 4, select = -disp)
  )

  driver$click("test-add_xform")
  wait_for_visible(driver, "test-xform_modal-apply")
  driver$set_inputs("test-xform_modal-select-select_cols" = c("mpg", "cyl", "hp"))
  driver$wait_for_idle()
  driver$click("test-xform_modal-apply")
  wait_for_not_exists(driver, "test-xform_modal-apply")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, subset = cyl == 4, select = c(mpg, cyl, hp))
  )

  expect_true(is_shiny_enabled(driver, "test-add_xform"))
  expect_true(is_shiny_enabled(driver, "test-continue"))
  expect_equal(driver$get_text("#test-error"), "")

  driver$set_inputs(
    `test-code-insert` = "2",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  wait_for_visible(driver, "test-xform_modal-apply")
  driver$set_inputs("test-xform_modal-select-select_cols" = c("vs", "am"))
  driver$wait_for_idle()
  driver$click("test-xform_modal-apply")
  wait_for_not_exists(driver, "test-xform_modal-apply")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, select = c(vs, am))
  )

  expect_false(is_shiny_enabled(driver, "test-add_xform"))
  expect_false(is_shiny_enabled(driver, "test-continue"))
  expect_match(driver$get_text("#test-error"), "cyl")

  expect_false(is_shiny_enabled(driver, "test-redo"))
  driver$click("test-undo")
  driver$wait_for_idle()
  expect_true(is_shiny_enabled(driver, "test-add_xform"))
  expect_true(is_shiny_enabled(driver, "test-continue"))
  expect_equal(driver$get_text("#test-error"), "")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, subset = cyl == 4, select = c(mpg, cyl, hp))
  )

  expect_true(is_shiny_enabled(driver, "test-redo"))
  driver$click("test-redo")
  driver$wait_for_idle()
  expect_false(is_shiny_enabled(driver, "test-add_xform"))
  expect_false(is_shiny_enabled(driver, "test-continue"))
  expect_match(driver$get_text("#test-error"), "cyl")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, select = c(vs, am))
  )

  driver$set_inputs(
    `test-code-modify` = "2",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  wait_for_visible(driver, "test-xform_modal-apply")
  driver$set_inputs("test-xform_modal-select-select_name" = "newdf")
  driver$set_inputs("test-xform_modal-select-select_cols" = c("cyl", "vs", "am"))
  driver$wait_for_idle()
  driver$click("test-xform_modal-apply")
  wait_for_not_exists(driver, "test-xform_modal-apply")
  expect_false(is_shiny_enabled(driver, "test-add_xform"))
  expect_false(is_shiny_enabled(driver, "test-continue"))
  expect_match(driver$get_text("#test-error"), "mpg")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, subset = cyl == 4, select = c(cyl, vs, am))
  )

  driver$set_inputs(
    `test-code-delete` = "4",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  expect_true(is_shiny_enabled(driver, "test-add_xform"))
  expect_true(is_shiny_enabled(driver, "test-continue"))
  expect_equal(driver$get_text("#test-error"), "")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, subset = cyl == 4, select = c(cyl, vs, am))
  )

  driver$set_inputs("test-use_tidyverse" = FALSE)
  driver$wait_for_idle()
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, subset = cyl == 4, select = c(cyl, vs, am))
  )

  driver$set_inputs(
    `test-table-missing` = "am",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  wait_for_visible(driver, "test-xform_modal-apply")
  driver$click("test-xform_modal-apply")
  wait_for_not_exists(driver, "test-xform_modal-apply")
  driver$expect_values(export = "code")
  expect_equal(
    driver$get_value(export = "result"),
    subset(mtcarsnew, subset = (cyl == 4) & !is.na(am), select = c(cyl, vs, am))
  )
})

test_that("page_xforms shiny module works when given a large static dataset", {
  app <- shinyApp(
    ui = fluidPage(
      assistdomino:::page_xforms_ui("test")
    ),
    server = function(input, output, session) {
      Sys.setenv(SHINYMIXPANEL_DISABLE = "1")

      assign("df1", data.frame(x = seq(10001)), envir = globalenv())

      res <- assistdomino:::page_xforms_server("test", data_name_in = "df1")
      exportTestValues(
        code = res$code(),
        result = res$result()
      )
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE)
  driver$wait_for_idle()

  wait_for_visible(driver, "test-main_section")
  expect_equal(driver$get_value(export = "code"), "df1")
  expect_identical(driver$get_value(export = "result"), data.frame(x = seq(DATASET_MAX_ROWS_TO_LOAD)))
})

test_that("page_xforms shiny module works when given a reactive dataset", {
  app <- shinyApp(
    ui = fluidPage(
      selectInput("data", "data", c("df1", "df2")),
      assistdomino:::page_xforms_ui("test")
    ),
    server = function(input, output, session) {
      Sys.setenv(SHINYMIXPANEL_DISABLE = "1")

      assign("df1", mtcars, envir = globalenv())
      assign("df2", iris, envir = globalenv())

      res <- assistdomino:::page_xforms_server("test", data_name_in = reactive(input$data))
      exportTestValues(
        code = res$code(),
        result = res$result()
      )
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE)
  driver$wait_for_idle()

  wait_for_visible(driver, "test-main_section")
  expect_equal(driver$get_value(export = "code"), "df1")
  expect_identical(driver$get_value(export = "result"), mtcars)

  driver$set_inputs(
    `test-table-drop` = "disp",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  expect_length(driver$get_value(export = "code"), 2)

  driver$set_inputs("data" = "df2")
  driver$wait_for_idle()
  expect_equal(driver$get_value(export = "code"), "df2")
  expect_identical(driver$get_value(export = "result"), iris)
})
