library(shiny)

skip_on_cran()

test_that("pairs_selector with no reactive lists works", {
  app <- shinyApp(
    fluidPage(
      assistdomino:::pairs_selector_ui("pairs", choices_1 = letters[1:5], choices_2 = LETTERS[1:5]),
      actionButton("reset", "reset"),
      actionButton("set", "set a=B,d=A"),
      verbatimTextOutput("out")
    ),
    function(input, output, session) {
      res <- assistdomino:::pairs_selector_server("pairs")
      output$out <- renderPrint({
        res$get()
      })
      observeEvent(input$reset, {
        res$set()
      })
      observeEvent(input$set, {
        res$set(list("a"="B", "d"="A"))
      })
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "pairs-selector-no-reactive")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "b", `pairs-choice_2` = "C")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "a", `pairs-choice_2` = "E")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "b", `pairs-choice_2` = "C")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(
    `pairs-remove_pair` = 2,
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(
    `pairs-remove_pair` = 1,
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "b", `pairs-choice_2` = "B")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$click("set")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "c", `pairs-choice_2` = "A")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$click("reset")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "d", `pairs-choice_2` = "A")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
})

test_that("pairs_selector with first list being reactive", {
  app <- shinyApp(
    fluidPage(
      assistdomino:::pairs_selector_ui("pairs", choices_2 = letters[1:5]),
      selectInput("data", "data", c("full", "small")),
      actionButton("reset", "reset"),
      actionButton("set", "set mpg=a,hp=d"),
      verbatimTextOutput("out")
    ),
    function(input, output, session) {
      data <- reactive({
        if (input$data == "full") mtcars
        else mtcars[, c("mpg", "cyl", "disp")]
      })
      res <- assistdomino:::pairs_selector_server("pairs", choices_df_1 = data)
      output$out <- renderPrint({
        res$get()
      })
      observeEvent(input$reset, {
        res$set()
      })
      observeEvent(input$set, {
        res$set(list("mpg"="a", "hp"="d"))
      })
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "pairs-selector-first-reactive")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "mpg", `pairs-choice_2` = "a")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "hp", `pairs-choice_2` = "b")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$click("reset")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$click("set")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "cyl", `pairs-choice_2` = "c")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(
    `pairs-remove_pair` = 2,
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs("data" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "disp", `pairs-choice_2` = "e")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs("data" = "full")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "drat", `pairs-choice_2` = "a")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "mpg", `pairs-choice_2` = "a")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(
    `pairs-remove_pair` = 1,
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = "out")
})

test_that("pairs_selector with second list being reactive", {
  app <- shinyApp(
    fluidPage(
      assistdomino:::pairs_selector_ui("pairs", choices_1 = letters[1:5]),
      selectInput("data", "data", c("full", "small")),
      actionButton("reset", "reset"),
      actionButton("set", "set a=mpg,d=hp"),
      verbatimTextOutput("out")
    ),
    function(input, output, session) {
      data <- reactive({
        if (input$data == "full") mtcars
        else mtcars[, c("mpg", "cyl", "disp")]
      })
      res <- assistdomino:::pairs_selector_server("pairs", choices_df_2 = data)
      output$out <- renderPrint({
        res$get()
      })
      observeEvent(input$reset, {
        res$set()
      })
      observeEvent(input$set, {
        res$set(list("a"="mpg", "d"="hp"))
      })
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "pairs-selector-second-reactive")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_2` = "mpg", `pairs-choice_1` = "a")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_2` = "hp", `pairs-choice_1` = "b")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$click("reset")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$click("set")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_2` = "cyl", `pairs-choice_1` = "c")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(
    `pairs-remove_pair` = 2,
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs("data" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_2` = "disp", `pairs-choice_1` = "e")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs("data" = "full")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_2` = "drat", `pairs-choice_1` = "a")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_2` = "mpg", `pairs-choice_1` = "a")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(
    `pairs-remove_pair` = 1,
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = "out")
})

test_that("pairs_selector with both lists being reactive", {
  app <- shinyApp(
    fluidPage(
      assistdomino:::pairs_selector_ui("pairs"),
      selectInput("data1", "data1", c("full", "small")),
      selectInput("data2", "data2", c("full", "small")),
      actionButton("reset", "reset"),
      actionButton("set", "set mpg=cyl,disp=mpg"),
      verbatimTextOutput("out")
    ),
    function(input, output, session) {
      data1 <- reactive({
        if (input$data1 == "full") mtcars
        else mtcars[, c("mpg", "cyl", "disp")]
      })
      data2 <- reactive({
        if (input$data2 == "full") mtcars
        else mtcars[, c("mpg", "cyl", "disp")]
      })
      res <- assistdomino:::pairs_selector_server("pairs", choices_df_1 = data1, choices_df_2 = data2)
      output$out <- renderPrint({
        res$get()
      })
      observeEvent(input$reset, {
        res$set()
      })
      observeEvent(input$set, {
        res$set(list("mpg"="cyl", "disp"="mpg"))
      })
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "pairs-selector-both-reactive")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$click("set")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$click("reset")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "mpg", `pairs-choice_2` = "disp")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "drat", `pairs-choice_2` = "am")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "mpg", `pairs-choice_2` = "disp")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "cyl", `pairs-choice_2` = "mpg")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(
    `pairs-remove_pair` = 2,
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data2" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data2" = "full")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "cyl", `pairs-choice_2` = "hp")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data2" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")

  driver$set_inputs(`pairs-choice_1` = "cyl", `pairs-choice_2` = "mpg")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data1" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data1" = "full")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs(`pairs-choice_1` = "hp", `pairs-choice_2` = "mpg")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
  driver$set_inputs("data1" = "small")
  driver$wait_for_idle()
  driver$expect_values(output = "out")
})
