library(shiny)

skip_on_cran()

test_that("xforms_table works", {
  app <- shinyApp(
    fluidPage(
      selectInput("data", "data", c("empty", "cars")),
      assistdomino:::xforms_table_ui("test"),
      verbatimTextOutput("drop_out"),
      verbatimTextOutput("missing_out"),
      verbatimTextOutput("filter_out")
    ),
    function(input, output, session) {
      data <- reactive({
        if (input$data == "empty") {
          data.frame()
        } else if (input$data == "cars") {
          df <- cars
          df[2, 2] <- NA
          df[3, 1] <- NA
          df
        }
      })
      res <- assistdomino:::xforms_table_server("test", data)

      output$drop_out <- renderPrint({
        print(res$drop())
      })
      output$missing_out <- renderPrint({
        print(res$missing())
      })
      output$filter_out <- renderPrint({
        print(res$filter())
      })
    }
  )
  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "xforms-table")

  driver$wait_for_idle()
  driver$wait_for_value(input = "test-table__reactable__page")
  expect_match(driver$get_text("#test-table"), "Data has no columns")

  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))

  driver$set_inputs(data = "cars")
  driver$wait_for_idle()
  driver$wait_for_value(input = "test-table__reactable__page")
  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))

  driver$set_inputs(
    `test-drop` = "speed",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))

  driver$set_inputs(
    `test-missing` = "dist",
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))

  driver$set_inputs(
    `test-filter` = list(col = "speed", val = "4"),
    allow_no_input_binding_ = TRUE, priority_ = "event"
  )
  driver$wait_for_idle()
  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))

  driver$run_js("$('i.col-drop-btn[data-col-name=\"dist\"]').click()")
  driver$wait_for_idle()
  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))

  driver$run_js("$('i.cell-filter-btn').get(1).click()")
  driver$wait_for_idle()
  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))

  driver$run_js("$('i.cell-filter-btn').get(4).click()")
  driver$wait_for_idle()
  driver$expect_values(output = c("drop_out", "missing_out", "filter_out"))
})
