library(shiny)

skip_on_cran()

test_that("snippet_edit_mode shiny module works", {
  app <- shinyApp(
    ui = fluidPage(
      selectInput("file", "file", c("none", "editable", "noedit")),
      assistdomino:::snippet_edit_mode_ui("test")
    ),
    server = function(input, output, session) {
      Sys.setenv(
        SHINYMIXPANEL_DISABLE = "1",
        DOMINO_IMPORTED_CODE_DIR = "",
        DOMINO_WORKING_DIR = getwd()
      )

      file <- reactive({
        if (input$file == "none") {
          NULL
        } else if (input$file == "editable") {
          "myfile.R"
        } else if (input$file == "noedit") {
          system.file("sample_snippets", "snippets", "ggplot2", "boxplots.R", package = "assistdomino")
        }
      })

      assistdomino:::snippet_edit_mode_server("test", selected_snippet = file, curwd = reactive("a/b"))
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "snippet_edit_mode")
  driver$wait_for_idle()

  withr::local_dir(driver$get_dir())
  writeLines("test file", "myfile.R")

  wait_for_visible(driver, "test-enable_edit")

  expect_true(is_shiny_visible(driver, "test-enable_edit"))
  expect_false(is_shiny_visible(driver, "test-disable_edit"))
  expect_false(is_shiny_visible(driver, "test-add"))
  expect_false(is_shiny_visible(driver, "test-edit"))
  expect_false(is_shiny_visible(driver, "test-delete"))

  driver$click("test-enable_edit")
  wait_for_visible(driver, "test-disable_edit")
  expect_false(is_shiny_visible(driver, "test-enable_edit"))
  expect_true(is_shiny_visible(driver, "test-disable_edit"))
  expect_true(is_shiny_visible(driver, "test-add"))
  expect_false(is_shiny_visible(driver, "test-edit"))
  expect_false(is_shiny_visible(driver, "test-delete"))

  driver$click("test-disable_edit")
  wait_for_visible(driver, "test-enable_edit")
  expect_true(is_shiny_visible(driver, "test-enable_edit"))
  expect_false(is_shiny_visible(driver, "test-disable_edit"))
  expect_false(is_shiny_visible(driver, "test-add"))
  expect_false(is_shiny_visible(driver, "test-edit"))
  expect_false(is_shiny_visible(driver, "test-delete"))

  driver$click("test-enable_edit")
  wait_for_visible(driver, "test-disable_edit")
  expect_false(is_shiny_visible(driver, "test-enable_edit"))
  expect_true(is_shiny_visible(driver, "test-disable_edit"))
  expect_true(is_shiny_visible(driver, "test-add"))
  expect_false(is_shiny_visible(driver, "test-edit"))
  expect_false(is_shiny_visible(driver, "test-delete"))

  driver$set_inputs(file = "editable")
  driver$wait_for_idle()
  wait_for_visible(driver, "test-edit")
  expect_true(is_shiny_visible(driver, "test-add"))
  expect_true(is_shiny_visible(driver, "test-edit"))
  expect_true(is_shiny_visible(driver, "test-delete"))
  expect_true(is_shiny_enabled(driver, "test-edit"))
  expect_true(is_shiny_enabled(driver, "test-delete"))

  driver$set_inputs(file = "none")
  driver$wait_for_idle()
  wait_for_not_visible(driver, "test-edit")
  expect_true(is_shiny_visible(driver, "test-add"))
  expect_false(is_shiny_visible(driver, "test-edit"))
  expect_false(is_shiny_visible(driver, "test-delete"))

  driver$set_inputs(file = "editable")
  driver$wait_for_idle()
  wait_for_visible(driver, "test-edit")
  expect_true(is_shiny_visible(driver, "test-add"))
  expect_true(is_shiny_visible(driver, "test-edit"))
  expect_true(is_shiny_visible(driver, "test-delete"))
  expect_true(is_shiny_enabled(driver, "test-edit"))
  expect_true(is_shiny_enabled(driver, "test-delete"))

  driver$click("test-disable_edit")
  wait_for_visible(driver, "test-enable_edit")
  expect_true(is_shiny_visible(driver, "test-enable_edit"))
  expect_false(is_shiny_visible(driver, "test-disable_edit"))
  expect_false(is_shiny_visible(driver, "test-add"))
  expect_false(is_shiny_visible(driver, "test-edit"))
  expect_false(is_shiny_visible(driver, "test-delete"))

  driver$set_inputs(file = "noedit")
  driver$wait_for_idle()
  driver$click("test-enable_edit")
  wait_for_visible(driver, "test-disable_edit")
  expect_false(is_shiny_visible(driver, "test-enable_edit"))
  expect_true(is_shiny_visible(driver, "test-disable_edit"))
  expect_true(is_shiny_visible(driver, "test-add"))
  expect_true(is_shiny_visible(driver, "test-edit"))
  expect_true(is_shiny_visible(driver, "test-delete"))
  expect_false(is_shiny_enabled(driver, "test-edit"))
  expect_false(is_shiny_enabled(driver, "test-delete"))

  driver$set_inputs(file = "editable")
  driver$wait_for_idle()
  wait_for_enabled(driver, "test-edit")
  expect_true(is_shiny_enabled(driver, "test-edit"))
  expect_true(is_shiny_enabled(driver, "test-delete"))

  driver$set_inputs(file = "noedit")
  driver$wait_for_idle()
  wait_for_disabled(driver, "test-edit")
  expect_false(is_shiny_enabled(driver, "test-edit"))
  expect_false(is_shiny_enabled(driver, "test-delete"))

  driver$set_inputs(file = "editable")
  driver$wait_for_idle()
  wait_for_enabled(driver, "test-edit")

  driver$click("test-add")
  wait_for_visible(driver, "test-snippet_modal-name")
  expect_true(is_shiny_enabled(driver, "test-snippet_modal-repo"))
  expect_true(is_shiny_enabled(driver, "test-snippet_modal-name"))
  expect_false(is_shiny_enabled(driver, "test-snippet_modal-add"))
  driver$set_inputs("test-snippet_modal-name" = "test5")
  driver$run_js("$(document.getElementById('test-snippet_modal-contents')).data('aceEditor').setValue('hello')")
  driver$wait_for_value(input = "test-snippet_modal-contents")
  expect_true(is_shiny_enabled(driver, "test-snippet_modal-add"))
  driver$click("test-snippet_modal-add")
  driver$run_js("swalService.close()")
  wait_for_not_exists(driver, "test-snippet_modal-name")
  expect_true(file.exists("snippets/a/b/test5.R"))
  expect_equal("hello", readLines("snippets/a/b/test5.R"))

  driver$click("test-edit")
  wait_for_visible(driver, "test-snippet_modal-name")
  wait_for_enabled(driver, "test-snippet_modal-add")
  expect_false(is_shiny_enabled(driver, "test-snippet_modal-repo"))
  expect_false(is_shiny_enabled(driver, "test-snippet_modal-name"))
  expect_true(is_shiny_enabled(driver, "test-snippet_modal-add"))
  expect_equal(driver$get_value(input = "test-snippet_modal-name"), "myfile")
  expect_equal("test file", readLines("myfile.R"))
  driver$run_js("$(document.getElementById('test-snippet_modal-contents')).data('aceEditor').setValue('')")
  wait_for_disabled(driver, "test-snippet_modal-add")
  driver$run_js("$(document.getElementById('test-snippet_modal-contents')).data('aceEditor').setValue('goodbye')")
  wait_for_enabled(driver, "test-snippet_modal-add")
  driver$click("test-snippet_modal-edit")
  driver$run_js("swalService.close()")
  wait_for_not_exists(driver, "test-snippet_modal-name")
  expect_equal(readLines("myfile.R"), "goodbye")

  driver$click("test-delete")
  driver$wait_for_js("$('.sweet-alert .confirm').is(':visible')")
  expect_true(file.exists("myfile.R"))
  Sys.sleep(0.1)
  driver$run_js("$('.sweet-alert .confirm').click()")
  driver$run_js("swalService.close()")

  skip_on_ci()

  expect_false(file.exists("myfile.R"))
})
