library(shiny)

skip_on_cran()

test_that("AggregateTransformation shiny", {
  app <- shinyApp(
    fluidPage(
      selectInput("dataset", "dataset", c("mtcars", "iris")),
      assistdomino:::AggregateTransformation$shiny$ui("test"),
      verbatimTextOutput("xform")
    ),
    function(input, output, session) {
      dataset <- reactive({
        get(input$dataset)
      })
      xform <- reactive({
        if (input$dataset == "iris") {
          assistdomino:::AggregateTransformation$new("Species", c(Sepal.Length = "sum", Sepal.Width = "mean"), name_out = paste0(input$dataset, "_df"))
        } else {
          assistdomino:::AggregateTransformation$new("cyl", c(mpg = "max"), name_out = paste0(input$dataset, "_df"))
        }
      })

      res <- assistdomino:::AggregateTransformation$shiny$server("test", dataset, xform)

      output$xform <- renderPrint({
        req(res$validate())
        res$xform()
      })
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "transformation-aggregate")
  driver$wait_for_idle()
  driver$wait_for_value(input = "test-aggregate_cols")
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_cols` = c("cyl", "am"))
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-choice_1` = "mpg")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-choice_2` = "sum")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-choice_1` = "drat")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-choice_2` = "sum")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-choice_2` = "sum")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-choice_1` = "mpg")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-remove_pair` = 3, allow_no_input_binding_ = TRUE, priority_ = "event")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-remove_pair` = 1, allow_no_input_binding_ = TRUE, priority_ = "event")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_pairs-remove_pair` = 1, allow_no_input_binding_ = TRUE, priority_ = "event")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")

  driver$set_inputs(dataset = "iris")
  driver$wait_for_idle()
  driver$wait_for_value(input = "test-aggregate_cols")
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_name` = "iris_df@")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
  driver$set_inputs(`test-aggregate_cols` = "Sepal.Width")
  driver$wait_for_idle()
  driver$expect_values(input = TRUE, output = "xform")
})
