library(shiny)

skip_on_cran()

test_that("MergeTransformation shiny", {
  df1 <- data.frame(A = letters[1:10], B = c(letters[c(3:10, 18, 19)]), X = c(1:8, 15, 20))
  df2 <- data.frame(A = letters[5:14], B = letters[7:16], Y = 1:10)
  df3 <- data.frame(A = letters[1:10], C = letters[10:19], Y = c(1:8, 15, 20))
  df4 <- data.frame(foo = "bar")

  app <- shinyApp(
    fluidPage(
      selectInput("dataset", "dataset", c("df1", "df2", "df3", "df4")),
      assistdomino:::MergeTransformation$shiny$ui("test"),
      verbatimTextOutput("xform")
    ),
    function(input, output, session) {
      assign("df1", df1, envir = globalenv())
      assign("df2", df2, envir = globalenv())
      assign("df3", df3, envir = globalenv())
      assign("df4", df4, envir = globalenv())

      dataset <- reactive({
        get(input$dataset)
      })
      res <- assistdomino:::MergeTransformation$shiny$server("test", dataset)

      output$xform <- renderPrint({
        req(res$validate())
        res$xform()
      })
    }
  )

  driver <- shinytest2::AppDriver$new(app, expect_values_screenshot_args = FALSE, name = "transformation-merge")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-df" = "df1")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-type" = "right")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-natural" = FALSE)
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-merge_pairs-choice_1" = "A", "test-merge_pairs-choice_2" = "A")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-merge_pairs-choice_1" = "B", "test-merge_pairs-choice_2" = "B")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-df" = "df2")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-df" = "df3")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-merge_pairs-choice_1" = "X", "test-merge_pairs-choice_2" = "Y")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-natural" = TRUE)
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-df" = "df2")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-natural" = FALSE)
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-df" = "df1")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-merge_pairs-choice_1" = "A", "test-merge_pairs-choice_2" = "B")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-merge_pairs-choice_1" = "X", "test-merge_pairs-choice_2" = "X")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs(`test-merge_pairs-remove_pair` = 1, allow_no_input_binding_ = TRUE, priority_ = "event")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))

  driver$set_inputs("dataset" = "df2")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-df" = "df3")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-type" = "outer")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-merge_pairs-choice_1" = "Y", "test-merge_pairs-choice_2" = "Y")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-natural" = TRUE)
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))
  driver$set_inputs("test-df" = "df2")
  driver$wait_for_idle()
  driver$expect_values(output = c("xform", "test-joining_on_natural"))

  driver$set_inputs("test-df" = "df4")
  wait_for_disabled(driver, "test-natural")

  driver$set_inputs("test-name" = "bad name")
  driver$wait_for_idle()
  expect_match(driver$get_value(input = "test-name"), "bad.name")
})
