test_that("MergeTransformation works", {
  df1 <- data.frame(A = letters[1:10], X = 1:10)
  df2 <- data.frame(A = letters[5:14], Y = 1:10)
  df3 <- data.frame(A = letters[1:10], B = c(letters[c(3:10, 18, 19)]), X = c(1:8, 15, 20))
  df4 <- data.frame(A = letters[5:14], B = letters[7:16], Y = 1:10)
  df5 <- data.frame(A = letters[1:10], C = letters[10:19], Y = c(1:8, 15, 20))

  mock_env <- new.env(parent = emptyenv())
  mock_env$df1 <- df1
  mock_env$df2 <- df2
  mock_env$df3 <- df3
  mock_env$df4 <- df4
  mock_env$df5 <- df5
  withr::local_environment(mock_env)

  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "inner")),
    merge(df1, df2)
  )
  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "inner", tidyverse = TRUE), quiet = TRUE),
    merge(df1, df2)
  )

  expect_identical(
    run_xform(df2, MergeTransformation$new("df1", "inner")),
    merge(df2, df1)
  )
  expect_identical(
    run_xform(df2, MergeTransformation$new("df1", "inner", tidyverse = TRUE), quiet = TRUE),
    merge(df2, df1)
  )

  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "outer")),
    merge(df1, df2, all = TRUE)
  )
  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "outer", tidyverse = TRUE), quiet = TRUE),
    merge(df1, df2, all = TRUE)
  )

  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "left")),
    merge(df1, df2, all.x = TRUE)
  )
  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "left", tidyverse = TRUE), quiet = TRUE),
    merge(df1, df2, all.x = TRUE)
  )

  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "right")),
    merge(df1, df2, all.y = TRUE)
  )
  expect_identical(
    run_xform(df1, MergeTransformation$new("df2", "right", tidyverse = TRUE), quiet = TRUE),
    merge(df1, df2, all.y = TRUE)
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner")),
    merge(df3, df4)
  )
  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", tidyverse = TRUE), quiet = TRUE),
    merge(df3, df4)
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("A" = "A"))),
    merge(df3, df4, by = "A")
  )
  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("A" = "A"), tidyverse = TRUE), quiet = TRUE),
    merge(df3, df4, by = "A")
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("B" = "B"))),
    merge(df3, df4, by = "B")
  )
  expect_identical(
    dplyr::relocate(
      run_xform(df3, MergeTransformation$new("df4", "inner", by = list("B" = "B"), tidyverse = TRUE), quiet = TRUE),
      "B"
    ),
    merge(df3, df4, by = "B")
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("A" = "B"))),
    merge(df3, df4, by.x = "A", by.y = "B")
  )
  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("A" = "B"), tidyverse = TRUE), quiet = TRUE),
    merge(df3, df4, by.x = "A", by.y = "B")
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("B" = "A"))),
    merge(df3, df4, by.x = "B", by.y = "A")
  )
  expect_identical(
    dplyr::relocate(
      run_xform(df3, MergeTransformation$new("df4", "inner", by = list("B" = "A"), tidyverse = TRUE), quiet = TRUE),
      "B"
    ),
    merge(df3, df4, by.x = "B", by.y = "A")
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df5", "inner", by = list("A" = "A", "X" = "Y"))),
    merge(df3, df5, by.x = c("A", "X"), by.y = c("A", "Y"))
  )
  expect_equal(
    dplyr::relocate(
      run_xform(df3, MergeTransformation$new("df5", "inner", by = list("A" = "A", "X" = "Y"), tidyverse = TRUE), quiet = TRUE),
      c("A", "X")
    ),
    merge(df3, df5, by.x = c("A", "X"), by.y = c("A", "Y"))
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("A" = "A", "B" = "B"))),
    merge(df3, df4, by.x = c("A", "B"), by.y = c("A", "B"))
  )
  expect_identical(
    run_xform(df3, MergeTransformation$new("df4", "inner", by = list("A" = "A", "B" = "B"), tidyverse = TRUE), quiet = TRUE),
    merge(df3, df4, by.x = c("A", "B"), by.y = c("A", "B"))
  )

  expect_identical(
    run_xform(df3, MergeTransformation$new("df5", "inner", by = list("B" = "C", "X" = "Y"))),
    merge(df3, df5, by.x = c("B", "X"), by.y = c("C", "Y"))
  )
  expect_identical(
    dplyr::relocate(
      run_xform(df3, MergeTransformation$new("df5", "inner", by = list("B" = "C", "X" = "Y"), tidyverse = TRUE), quiet = TRUE),
      c("B", "X")
    ),
    merge(df3, df5, by.x = c("B", "X"), by.y = c("C", "Y"))
  )
})

test_that("MergeTransformation parameters error correctly", {
  expect_error(MergeTransformation$new())
  expect_error(MergeTransformation$new("test"))
  expect_error(MergeTransformation$new("test", "test"))
  expect_error(MergeTransformation$new("test", "inner"), NA)
  expect_error(MergeTransformation$new("test", "inner", list("a")))
  expect_error(MergeTransformation$new("test", "inner", c("a")))
  expect_error(MergeTransformation$new("test", "inner", list("a", "b")))
  expect_error(MergeTransformation$new("test", "inner", c("a", "b")))
  expect_error(MergeTransformation$new("test", "inner", list("a" = "A")), NA)
  expect_error(MergeTransformation$new("test", "inner", c("a" = "A")), NA)
  expect_error(MergeTransformation$new("test", "inner", list("a" = "A", "b")))
  expect_error(MergeTransformation$new("test", "inner", c("a" = "A", "b")))
  expect_error(MergeTransformation$new("test", "inner", list("a" = "A", "b" = "B")), NA)
  expect_error(MergeTransformation$new("test", "inner", c("a" = "A", "b" = "B")), NA)
})

test_that("MergeTransformation printing", {
  expect_output(
    (MergeTransformation$new("mydf", "right"))$print(),
    "<MergeTransformation>.*df.*base.*right.*mydf.*natural"
  )
  expect_output(
    (MergeTransformation$new("mydf", "inner", tidyverse = TRUE))$print(),
    "<MergeTransformation>.*df.*tidyverse.*inner.*mydf.*natural"
  )
  expect_output(
    (MergeTransformation$new("mydf", "outer", name_out = "newdf"))$print(),
    "<MergeTransformation>.*newdf.*base.*outer.*mydf.*natural"
  )
  expect_output(
    (MergeTransformation$new("mydf", "left", list("col1" = "col1", "col2" = "col2")))$print(),
    "<MergeTransformation>.*df.*base.*left.*mydf.*: col1, col2"
  )
  expect_output(
    (MergeTransformation$new("mydf", "left", list("col1" = "COL1", "col2" = "COL2")))$print(),
    "<MergeTransformation>.*df.*base.*left.*mydf.*: col1=COL1, col2=COL2"
  )
  expect_output(
    (MergeTransformation$new("mydf", "left", list("col1" = "COL1", "col2" = "col2", "col3" = "COL3")))$print(),
    "<MergeTransformation>.*df.*base.*left.*mydf.*: col2, col1=COL1, col3=COL3"
  )
})
